REBOL [
	Title: "REBOL 3 Boot Sys: Encoder and Decoder"
	Rights: "Copyright REBOL Technologies 2010"
	File: %sys-codec.r
	Context: sys
	Note: {
		The boot binding of this module is SYS then LIB deep.
		Any non-local words not found in those contexts WILL BE
		UNBOUND and will error out at runtime!
	}
]

decode: funct [
 	{Decodes a series of bytes into the related datatype (e.g. image!).}
	type [word!] {Media type (jpeg, png, etc.)}
	data [binary!] {The data to decode}
][
	unless all [
		cod: select system/codecs type
		data: do-codec cod/entry 'decode data
	][
		cause-error 'access 'no-codec type
	]
	data
]

encode: funct [
	{Encodes a datatype (e.g. image!) into a series of bytes.}
	type [word!] {Media type (jpeg, png, etc.)}
	data [image! binary! string!] {The data to encode}
	/options opts [block!] {Special encoding options}
][
	unless all [
		cod: select system/codecs type
		data: do-codec cod/entry 'encode data
	][
		cause-error 'access 'no-codec type
	]
	data
]

encoding?: funct [
	"Returns the media codec name for given binary data. (identify)"
	data [binary!]
][
	foreach [name codec] system/codecs [
		if do-codec codec/entry 'identify data [
			return name
		]
	]
	none
]

export [decode encode encoding?]